# Bugzilla plugin for rbot
# Copyright (c) 2005-2007 Diego Pettenò <flameeyes@gmail.com>
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU Affero General Public License as
#  published by the Free Software Foundation, either version 3 of the
#  License, or (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU Affero General Public License for more details.
#
#  You should have received a copy of the GNU Affero General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.

require 'httpclient'

class BugzillaPlugin < Plugin

  def summary(zilla, bugno)
    # OpenOffice's issuezilla is tricky, they call it issue_status
    status_txt = "bug_status"

    require 'uri'

    zilla[:client] = HTTPAccess2::Client.new(nil, zilla[:useragent]) unless zilla[:client]
    zilla[:client].ssl_config.verify_mode = OpenSSL::SSL::VERIFY_NONE

    bugdata = REXML::Document.new(zilla[:client].get_content(zilla[:dataurl].gsub('#{bugno}', bugno)))

    return "Unable to load bug # #{bugno}" if not bugdata

    bugxml = bugdata.root.get_elements("bug")[0]
    unless bugxml
      bugxml = bugdata.root.get_elements("issue")[0]
      status_txt = "issue_status"
    end

    return "Unable to parse bug # #{bugno}" unless bugxml

    return "Bug # #{bugno} not found" if bugxml.attribute("status_code").to_s == "404" or bugxml.attribute("error").to_s.downcase == "notfound"
      bugxml.attribute("error")

    if bugxml.get_text("product") == "" or bugxml.get_text("product") == nil
      product_component = bugxml.get_text("component")
    elsif bugxml.get_text("component") == "" or bugxml.get_text("component") == nil
      product_component = bugxml.get_text("product")
    else
      product_component = "#{bugxml.get_text("product")} | #{bugxml.get_text("component")}"
    end

    if bugxml.get_text("resolution") == ""
      status = bugxml.get_text(status_txt)
    else
      status = "#{bugxml.get_text(status_txt)}, #{bugxml.get_text("resolution")}"
    end

    return "Bug #{bugno}; \"#{bugxml.get_text("short_desc")}\"; #{product_component}; #{status}; #{bugxml.get_text("reporter")} -> #{bugxml.get_text("assigned_to")}; #{zilla[:reporturl].gsub('#{bugno}', bugno)}"
  end

  def initialize
    super
    @zillas = @registry[:zillas] if @registry.has_key?(:zillas)

    @zillas = {} unless @zillas

  end

  def save
    @registry[:zillas] = @zillas
  end

  def help(plugin, topic = "")
    case plugin
    when "bug"
      return "bug #{Bold}bugzilla#{Bold} #{Bold}number#{Bold} : show the data about given bugzilla's bug."
    when "zilla"
      case topic
      when "add"
        return "zilla add #{Bold}name#{Bold} #{Bold}dataurl#{Bold} #{Bold}reporturl#{Bold} [#{Bold}useragent#{Bold}] : adds a new bugzilla (use \#{bugno} in URLs to replace the bug number)"
      when "remove"
        return "zilla remove #{Bold}name#{Bold} : removes the named bugzilla"
      when "list"
        return "zilla list : shows current managed bugzillas"
      when "show"
        return "zilla show #{Bold}name#{Bold} : shows the configuration for the named bugzilla."
      when "source"
        return "zilla source : shows a link to the plugin's sources."
      when "credits"
        return "zilla credits : shows the plugin's credits and license."
      else
        return "zilla #{Bold}add#{Bold}|#{Bold}remove#{Bold}|#{Bold}list#{Bold}|#{Bold}show#{Bold}|#{Bold}source#{Bold}|#{Bold}credits#{Bold} : manages bugzilla lists."
      end
    end
  end

  def bug(m, params)
    unless @zillas.has_key?(params[:zilla])
      m.reply "Bugzilla #{params[:zilla]} not found."
      return
    end

    unless params[:number] =~ /^#?(\d+)$/
      m.reply "Bug #{params[:number]} not valid."
      return
    end

    m.reply summary(@zillas[params[:zilla]], $1)
  end

  def add_zilla(m, params, force = false)
    if @zillas.has_key?(params[:name]) && !force
      m.reply "Bugzilla #{params[:name]} already present."
      return
    end

    unless params[:dataurl].match(/https?/)
      m.reply "I only deal with data from HTTP sources, so I can't use #{url}"
    end

    @zillas[params[:name]] = params
    m.reply "Added #{params[:name]}"
  end

  def remove_zilla(m, params)
    @zillas.delete(params[:name]) { |name|
      m.reply "Bugzilla #{name} not found."
      return
    }

    m.okay
  end

  def list_zilla(m, params)
    m.reply @zillas.keys.join(", ")
  end

  def show_zilla(m, params)
    unless @zillas.has_key?(params[:name])
      m.reply "Bugzilla #{params[:name]} not found."
      return
    end

    m.reply @zillas[params[:name]].inspect
  end

  def link_source(m, params)
    m.reply "http://flameeyes.is-a-geek.org/projects#rbot-bugzilla"
  end

  def show_credits(m, params)
    m.reply "Copyright (C) 2005-2007 Diego Pettenò. Distributed under Affero General Public License version 3."
  end
end

plugin = BugzillaPlugin.new

plugin.default_auth( 'modify', false )
plugin.default_auth( 'view', true )

plugin.map 'bug :zilla :number',
  :action => 'bug',
  :auth_path => 'view'

plugin.map 'zilla add :name :dataurl :reporturl :useragent',
  :action => 'add_zilla',
  :defaults => { :useragent => "rBot/#{$version}" },
  :auth_path => 'modify'

plugin.map 'zilla remove :name',
  :action => 'remove_zilla',
  :auth_path => 'modify'

plugin.map 'zilla list',
  :action => 'list_zilla',
  :auth_path => 'view'

plugin.map 'zilla show :name',
  :action => 'show_zilla',
  :auth_path => 'view'

plugin.map 'zilla source',
  :action => 'link_source',
  :auth_path => 'view'

plugin.map 'zilla credits',
  :action => 'show_credits',
  :auth_path => 'view'
